<?
// 
// Clok/PHP
//    by Steve Babineau <iam@nkrm.org>
//
// Based on Clok by Caskey L. Dickson <caskey@technocage.com>
//         http://www.technocage.com/~caskey/
//
// PURPOSE: To display a cool looking clock. Check out
//          ( http://www.technocage.com/~caskey/clok/ ) for an explanation.
//
// LICENSE: GNU GPL Version 2 ( http://www.gnu.org/licenses/gpl.html )
//
// CHANGELOG:
//
//  1.3: Added timezone functionality. Now you can show a clok for any
//       timezone.
//
//  1.2: Incorporated into the clok distribution.  Minor changes to
//       improve formatting with 80 column limit --cld
//
//  1.1: Added all the query-settable variables. I need me a shower!
//  
//  1.0: Initial release. Stimulating, isn't it?
//       - Doesn't support different timezones though. Ya gotta have
//         SOMEthing to look forward to!
//
// REQUIREMENTS:
//
//  - PHP 4.0.6+
//  - GD 2.0.1+
//  - Beer (preferrably Guinness)
//
// USAGE: Call clok.php from your HTML page as an image, like this:
//
//  <img src="clok.php">
//
//  Of course you can add any HTML based options you like, like height, width,
//  border, style, etc.
//
//  Also, everything you can configure in here you can pass as part of the
//  query string. You're welcome to set them in either/both places, but the ones
//  from the query string take precedence.
//
//    w  -  image width
//
//    h  -  image height
//
//    p  -  profile name
//
//              The profiles are used to control the colors on the bar.
//     NOTE: Make sure you specify the profile without the "prof_"!
//
//    mw -  marker width
//
//    mc -  marker color
//
//    bc -  border color
//
//    c1 -      color 1 (the darkest)
//
//    c2 -      color 2 (the second to the darkest)
//
//    c3 -      color 3 (the second to the lightest)
//
//    c4 -      color 4 (the lightest)
//
//    bcb -     blend cell borders? (1 = true or 0 = false)
//
//    cb  -     cell borders? (1 = true or 0 = false)
//
//    ib -      image borders? (1 = true or 0 = false)
//
//    tz -      timezone. Make sure you pick one that your system supports
//    (any one from the included timezones.txt file should do)
//
//  Example:
//
//    <img
//       src="clok.php?w=500&h=100&p=human&mw=10&mc=00FF00&bc=FF0000& 
//                     c1=222222&c2=444444&c3=888888&c4=AAAAAA&bcb=1&
//                     cb=1&ib=1tz=America/Los_Angeles"
//       border="0">
//

// --- Start Config
//   Change these to your liking.. the variable names are pretty self
//   explanatory. If you don't get one, change it and see what happens :)

$default_image_border = 0;
$default_cell_border = 0;
$default_blend_cell_borders = 1;

// colors go from dark to light
$default_color1 = "000000";
$default_color2 = "303088";
$default_color3 = "6060CC";
$default_color4 = "9090FF";
$default_border_color = "C0C0C0";
$default_marker_color = "FF0000";

$default_marker_width = 1;

$default_width = 770;
$default_height = 17;

$default_prof = "default";

// Set the default timezone here. Make sure you pick one that your system
// supports (any one from the included timezones.txt file should do)
// If it's empty (""), it'll just use the system's timezone.
$default_timezone = "";

// profiles go here
// NOTE: You can add as many as you like, but make sure they're named
//       "$prof_<something>" and have exactly 24 values ranging from 1 to 4
//       (darkest to lightest).
$prof_default = array(2, 2, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 
                      4, 4, 3, 3, 3, 3, 2, 2, 2, 2, 2, 2);
$prof_human   = array(1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 
                      4, 3, 3, 3, 3, 2, 2, 2, 2, 2, 2, 1);
$prof_hacker  = array(2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 3,
                      3, 3, 3, 3, 4, 3, 3, 3, 3, 3, 2, 2);
$prof_server  = array(2, 2, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3,
                      4, 4, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2);
$prof_steve   = array(2, 1, 1, 1, 1, 1, 2, 3, 3, 3, 3, 3,
                      3, 3, 3, 4, 4, 4, 4, 3, 3, 3, 2, 2);

$timezone_file = "timezones.txt";

// --- End Config
// --- Start Main

// -start setting all the defaults

if (!empty($HTTP_GET_VARS["h"]))
  $im_h = $HTTP_GET_VARS["h"];
else
  $im_h = $default_height;

if (!empty($HTTP_GET_VARS["w"]))
  $im_w = $HTTP_GET_VARS["w"];
else
  $im_w = $default_width;

if ((!empty($HTTP_GET_VARS["p"])) && (!empty( ${prof_.$HTTP_GET_VARS["p"]} )))
  $prof = "prof_".$HTTP_GET_VARS["p"];
else 
  $prof = "prof_".$default_prof;

if (!empty($HTTP_GET_VARS["mw"]))
  $marker_width = $HTTP_GET_VARS["mw"];
else
  $marker_width = $default_marker_width;

if (!empty($HTTP_GET_VARS["mc"]))
  $marker_color = $HTTP_GET_VARS["mc"];
else
  $marker_color = $default_marker_color;

if (!empty($HTTP_GET_VARS["bc"]))
  $border_color = $HTTP_GET_VARS["bc"];
else
  $border_color = $default_border_color;

if (!empty($HTTP_GET_VARS["c1"]))
  $color1 = $HTTP_GET_VARS["c1"];
else
  $color1 = $default_color1;

if (!empty($HTTP_GET_VARS["c2"]))
  $color2 = $HTTP_GET_VARS["c2"];
else
  $color2 = $default_color2;

if (!empty($HTTP_GET_VARS["c3"]))
  $color3 = $HTTP_GET_VARS["c3"];
else
  $color3 = $default_color3;

if (!empty($HTTP_GET_VARS["c4"]))
  $color4 = $HTTP_GET_VARS["c4"];
else
  $color4 = $default_color4;

if (!empty($HTTP_GET_VARS["tz"]))
  $timezone = $HTTP_GET_VARS["tz"];
else
  $timezone = $default_timezone;

if (isset($HTTP_GET_VARS["bcb"]) && strlen($HTTP_GET_VARS["bcb"]) > 0)
  $blend_cell_borders = $HTTP_GET_VARS["bcb"];
else
  $blend_cell_borders = $default_blend_cell_borders;

if (isset($HTTP_GET_VARS["cb"]) && strlen($HTTP_GET_VARS["cb"]) > 0)
  $cell_border = $HTTP_GET_VARS["cb"];
else
  $cell_border = $default_cell_border;

if (isset($HTTP_GET_VARS["ib"]) && strlen($HTTP_GET_VARS["ib"]) > 0)
  $image_border = $HTTP_GET_VARS["ib"];
else
  $image_border = $default_image_border;

// -end setting all the defaults

// input timezones
$die_str = "Ack! I couldn't open the timezone file \"$timezone_file\"!".
     " Fix this!";
$fp = fopen($timezone_file, "r") or die($die_str);

$tz = array();

while ($tzline = fgets($fp, 1024))
{
        $tzline_array = explode("\t", $tzline);
        array_push($tz, $tzline_array);
}

fclose($fp);

header("Content-type: image/png");

$im = imageCreateTrueColor($im_w, $im_h);

$cell_width = $im_w / 24;

$border_clr = make_color($im, $border_color);
$marker_clr = make_color($im, $marker_color);
$clr1 = make_color($im, $color1);
$clr2 = make_color($im, $color2);
$clr3 = make_color($im, $color3);
$clr4 = make_color($im, $color4);

if ($blend_cell_borders) {
  $clr1_1 = make_color($im, $color1, 75);
  $clr2_1 = make_color($im, $color2, 75);
  $clr3_1 = make_color($im, $color3, 75);
  $clr4_1 = make_color($im, $color4, 75);

  $clr1_2 = make_color($im, $color1, 50);
  $clr2_2 = make_color($im, $color2, 50);
  $clr3_2 = make_color($im, $color3, 50);
  $clr4_2 = make_color($im, $color4, 50);

  $clr1_3 = make_color($im, $color1, 25);
  $clr2_3 = make_color($im, $color2, 25);
  $clr3_3 = make_color($im, $color3, 25);
  $clr4_3 = make_color($im, $color4, 25);

  imageAlphaBlending($im, true);
}

$y = 0;

for ($x = 0; $x < $im_w; $x = $x + $cell_width) {
  imageFilledRectangle($im, $x, 0, ($x + $cell_width),
      $im_h, ${clr.${$prof}[$y]});
  if ($blend_cell_borders) {
    if ($y > 0) {
      if (${$prof}[$y] != ${$prof}[$y-1]) {
        imageLine($im, $x, 0, $x, $im_h,
          ${clr.${$prof}[$y]._3});
        imageLine($im, $x-1, 0, $x-1, $im_h,
          ${clr.${$prof}[$y]._2});
        imageLine($im, $x-2, 0, $x-2, $im_h,
          ${clr.${$prof}[$y]._1});
      }
    }
  }
  
  if ($cell_border)
    imageLine($im, $x, 0, $x, $im_h, $border_clr);

  $y++;
}

if (!empty($timezone))
  putenv("TZ=$timezone");

$now = getdate();
$hour = $now["hours"];
$minutes = $now["minutes"];
$seconds = $now["seconds"];

$marker_pos = ($now["hours"] * $cell_width) +
  ($now["minutes"] * ($cell_width / 60)) +
  ($now["seconds"] * ($cell_width / 3600));

if ($marker_width < 1)
  $marker_width = 1;

imageFilledRectangle($im,
  $marker_pos, 0,
  $marker_pos+$marker_width-1, $im_h-1,
  $marker_clr);

if ($image_border)
  imageRectangle($im, 0, 0, $im_w-1, $im_h-1, $border_clr);

imagePNG($im);
imageDestroy($im);

function make_color($im, $color, $alpha = FALSE)
{
  $color = str_replace("#", "", $color);

  $red = hexdec(substr($color, 0, 2));
  $green = hexdec(substr($color, 2, 2));
  $blue = hexdec(substr($color, 4, 2));

  if (!$alpha)
    return(imageColorAllocate($im, $red, $green, $blue));
  else
    return(imageColorResolveAlpha($im, $red, $green, $blue, $alpha));

}
